comment ~
*************************************************************************
*	This is the vesa video library ver. 1.0 by Chris Hazard		*
*	All routines were written only by my own two hands, my mind,	*
*	and vesa documentation.  This file, along with go.asm may be	*
*	freely distributed, and used by whoever needs them.  I designed	*
*	these routines to work under Dieter Pawelczak's Pro32 DOS	*
*	Extender.  I have tested them on many computers, and it has	*
*	worked with every one (except an old laptop).  The ones that	*
*	need to be optimized are (fairly well).  I have tried to make	*
*	them easy to use, and have provided an example program (go.asm).*
*	This currently does not support flat mode vesa addressing,	*
*	because relatively few video cards support it (look for it in	*
*	future edditions).  There is, however, a faster way to 		*
*	bankswitch in real mode, but it gets slowed down too much in	*
*	protected (it takes too long for the processor to switch between*
*	the 2 modes).  The documentation for these routines can be found*
*	in vdoc.txt.  Note: this may not work for mode 13h - it hasn't	*
*	been tested (that mode is relatively easy to use though).	*
*	all testing has been using Borland's turbo assembler		*
*	Side note: This summer, I am planning on building my own	*
*	computer for college, and I will probably get a good graphic	*
*	accellerator card, so I'll release a new version of this to work*
*	on those cards.							*
*									*
*	-Chris Hazard		chazard@hartford.k12.wi.us		*
*	please e-mail me if you have any problems, comments, questions	*
*	this e-mail will be good until June 10 (?) 1998			*
*	I will post my new e-mail when I get it (it will be after 5/10)	*
*************************************************************************
~

ALIGN 4
ScreenBuffer    dw		(?)
NoVesa		db		"VESA not detected", 0
NotDetected	db		"Video Mode Not Detected", 0
BankError	db		"An error occured in bank switching",0
AllocErr	db		"Insufficient Memory Resources",0
BankShift	db		(?)
VideoSelector	dw		(?)
ModeToSet	dd		(?)

;Real Mode Calling "Structure"   
ALIGN 4
	Redi	dd	(?)
	Resi	dd	(?)
	Rebp	dd	(?)
		dd	0
	Rebx	dd	(?)
	Redx	dd	(?)
	Recx	dd	(?)
	Reax	dd	(?)
	Rflags	dw	(?)
	Res	dw	(?)
	Rds	dw	(?)
	Rfs	dw	(?)
	Rgs	dw	(?)
	Rip	dw	(?)
	Rcs	dw	(?)
	Rsp	dw	0
	Rss	dw	0
	Dummy	dw	(?)
;----------------------------------------------------------------------------


;****************************************************************************
Initialize proc
		call GetVesaInfo		;see if vesa is there
		call GetVesaModeInfo		;get info
		call SetVesaMode		;set mode
	;allocate memory for screenbuffer
		xor eax,eax	;Function 0
		mov cx,1	;Allocate 1 Descriptor
		int 31h
		mov [ScreenBuffer],ax
		mov ax,0501h	;Allocate Memory
		xor ecx,ecx
		mov bx,5	;010000h Bytes of Memory
		int 31h		;DPMI CALL
		jnc AOK

		call ReturnMode
		mov edi,offset AllocErr
		mov ah,1
		xor ebx,ebx
		xor eax,eax
		xor edx,edx
		call PrintError
		mov ax,4c00h
		int 21h
	AOK:

		mov ax,0007h	;Function 7:Set Basis Address
		mov dx,cx	;Low Part of Linear Address
		mov cx,bx	;upper Part of Linear Address
		mov bx,[ScreenBuffer]
		int 31h		;DPMI CALL
		mov bx,[ScreenBuffer]
		mov ax,0008h	;Function 8:Set Limit of descriptor
		mov cx,5h
		mov dx,0
		int 31h		;set Limit

		ret
ENDP

;****************************************************************************
GetVesaInfo proc NEAR

		mov Reax,4f00h
		mov Recx,0
		mov Redx,0
		mov Redi,0
		mov ax,word ptr cs:[8]
		mov Res,ax
		mov edi,0
		push es
		push ds
		pop es
		mov eax,300h
		mov ebx,10h
		xor ecx,ecx
		mov edi,offset Redi
		int 31h
		mov eax,[Reax]
		pop es
		cmp ax,4fh
		jz AllSet
		mov edi,offset NoVesa
		xor ebx,ebx
		xor ecx,ecx
		xor edx,edx
		mov ah,1
		call PrintError
		mov ax,4c00h
		int 21h
	AllSet:	ret
ENDP



;****************************************************************************
GetVesaModeInfo proc
		mov eax,[ModeToSet]
		mov Recx,eax
		mov ax,word ptr cs:[8]
		mov Reax,4f01h
		mov Res,ax
		mov Redi,0
		mov Redx,0
		mov ax,300h
		mov bx,10h
		xor ecx,ecx
		push es
		push ds
		pop es
		mov edi,offset Redi
		int 31h
		
		mov cx,cs:[6]
		mov es,cx
		mov ax,es:[0]
		and ax,1
		cmp ax,1
		je AllGood

		;show error in video mode
		pop es
		mov edi,offset NotDetected
		mov ah,1
		xor ecx,ecx
		xor ebx,ebx
		xor edx,edx
		call PrintError
		mov ax,4c00h
		int 21h
	AllGood:

		mov bx,es:[4]		;load Window Granularity
		mov ax,es:[6]		;load Window Size
		xor dx,dx
		div bx			;WinSize / WinGran

		cmp ax,1
		jne NB1
		mov [BankShift],0
		jmp NBF
	NB1:	cmp ax,2
		jne NB2
		mov [BankShift],1
		jmp NBF
	NB2:	cmp ax,4
		jne NB3
		mov [BankShift],2
	NB3:	cmp ax,8
		jne NB4
		mov [BankShift],3
		jmp NBF
	NB4:	cmp ax,16
		jne NB5
		mov [BankShift],4
		jmp NBF
	NB5:	cmp ax,32
		jne NB6
		mov [BankShift],5
		jmp NBF
	NB6:	cmp ax,64
		jne NBE
		mov [BankShift],6
		jmp NBF
	NBE:	;print stuff about error
		pop es
		mov edi,offset BankError
		mov ah,1
		xor ebx,ebx
		xor eax,eax
		xor edx,edx
		call PrintError
		mov ax,4c00h
		int 21h
	NBF:

	;Now save selector for video
		pop es
		mov BX,0A000h		;Video Graphic Memory
		mov AX,0002h
		int 31h
		mov [VideoSelector],ax
		ret
ENDP

;****************************************************************************
SetVesaMode proc
		xor eax,eax
		xor ebx,ebx
		xor edx,edx
		mov ax,4f02h
		mov ebx,[ModeToSet]
		xor ecx,ecx
		int 10h
		ret
ENDP


;****************************************************************************
ReturnMode proc
		mov ax,3
		int 10h
		ret
ENDP


;****************************************************************************
FlushSBuffer proc
		push es
		xor esi,esi
		push ds
		mov ax,[VideoSelector]
		mov bx,[ScreenBuffer]
		mov es,ax		;load video selector
		mov ds,bx
		xor edx,edx		;clear bankswitch
		xor ebx,ebx
		xor eax,eax

 FlushLoop:	mov cl,cs:[BankShift]
		push edx
		shl edx,cl		;calculate bank shift
		push eax
		mov ax,4f05h
		int 10h			;call vesa bank switch

		mov edi,0		;start at beginning of screen
		mov ecx,4000h		;loop this many times (DW = 4 B)
	rep	movsd			;flush screen buffer

		pop eax
		pop edx
		inc eax
		inc edx
		cmp eax,5
		jb cs:FlushLoop

		pop ds
		pop es

		ret
ENDP


;****************************************************************************
;SetPal		Sets the palette
;	takes location in ds:esi
SetPal proc
		mov dx,3DAh
	l1:
		in al,dx
		and al,08h
		jnz l1
	l2:
		in al,dx
		and al,08h
		jz l2

		mov al,0
		mov dx,3C8h
		out dx,al
		mov dx,3C9h
		mov ecx,768
		cld
	rep	outsb
		ret
ENDP


;****************************************************************************
;ClearScreenBuffer - does just what it says: puts everything to black
ClearScreenBuffer proc
	mov edi,0		;get address of screen buffer
	mov dx,[ScreenBuffer]
	push es
	mov es,dx
	mov eax,2f2f2f2fh	;move black to eax
	mov ecx,12c00h		;move 1/2 of the number of dwords of screen
	cld			;make sure we're in the right direction
   rep	stosd			;repeat for 1st half of screen

	pop es
	ret			;all done
ENDP

;****************************************************************************

PrintError proc
  ; ah : Color, cx : X, dx : Y, EDI: OFFSET Message
	mov bh,ah
	mov ax,cs:[2]
	mov es,ax
	mov ax,160
	mul dx
	shl cx,1
	add ax,cx
	mov si,ax        ; Screen Offset
printeloop:
	mov bl,[edi]
	cmp bl,0
	je printeende
	mov es:[si],bx
	add si,2
	inc edi
	jne printeloop
printeende:
	inc edi
	ret
ENDP
